@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -146,29 +146,27 @@ bool PFS_table_context::initialize(void)
   {
     /* Restore context from TLS. */
     PFS_table_context *context= static_cast<PFS_table_context *>(my_get_thread_local(m_thr_key));
-    DBUG_ASSERT(context != NULL);
+    assert(context != NULL);
 
     if(context)
     {
       m_last_version= context->m_current_version;
       m_map= context->m_map;
-      DBUG_ASSERT(m_map_size == context->m_map_size);
+      assert(m_map_size == context->m_map_size);
       m_map_size= context->m_map_size;
-      m_word_size= context->m_word_size;
     }
   }
   else
   {
     /* Check that TLS is not in use. */
     PFS_table_context *context= static_cast<PFS_table_context *>(my_get_thread_local(m_thr_key));
-    //DBUG_ASSERT(context == NULL);
+    //assert(context == NULL);
 
     context= this;
 
     /* Initialize a new context, store in TLS. */
     m_last_version= m_current_version;
     m_map= NULL;
-    m_word_size= sizeof(ulong) * 8;
 
     /* Allocate a bitmap to record which threads are materialized. */
     if (m_map_size > 0)
@@ -190,7 +188,7 @@ bool PFS_table_context::initialize(void)
 /* Constructor for global or single thread tables, map size = 0.  */
 PFS_table_context::PFS_table_context(ulonglong current_version, bool restore, thread_local_key_t key) :
                    m_thr_key(key), m_current_version(current_version), m_last_version(0),
-                   m_map(NULL), m_map_size(0), m_word_size(sizeof(ulong)),
+                   m_map(NULL), m_map_size(0),
                    m_restore(restore), m_initialized(false), m_last_item(0)
 {
   initialize();
@@ -199,7 +197,7 @@ PFS_table_context::PFS_table_context(ulonglong current_version, bool restore, th
 /* Constructor for by-thread or aggregate tables, map size = max thread/user/host/account. */
 PFS_table_context::PFS_table_context(ulonglong current_version, ulong map_size, bool restore, thread_local_key_t key) :
                    m_thr_key(key), m_current_version(current_version), m_last_version(0),
-                   m_map(NULL), m_map_size(map_size), m_word_size(sizeof(ulong)),
+                   m_map(NULL), m_map_size(map_size),
                    m_restore(restore), m_initialized(false), m_last_item(0)
 {
   initialize();
@@ -220,7 +218,7 @@ void PFS_table_context::set_item(ulong n)
     return;
   ulong word= n / m_word_size;
   ulong bit= n % m_word_size;
-  m_map[word] |= (1 << bit);
+  m_map[word] |= (1UL << bit);
   m_last_item= n;
 }
 
@@ -228,7 +226,7 @@ bool PFS_table_context::is_item_set(ulong n)
 {
   ulong word= n / m_word_size;
   ulong bit= n % m_word_size;
-  return (m_map[word] & (1 << bit));
+  return (m_map[word] & (1UL << bit));
 }
 
 
@@ -364,17 +362,15 @@ ha_rows PFS_engine_table_share::get_row_count(void) const
 int PFS_engine_table_share::write_row(TABLE *table, const unsigned char *buf,
                                       Field **fields) const
 {
-  my_bitmap_map *org_bitmap;
-
   if (m_write_row == NULL)
   {
     return HA_ERR_WRONG_COMMAND;
   }
 
   /* We internally read from Fields to support the write interface */
-  org_bitmap= dbug_tmp_use_all_columns(table, table->read_set);
+  MY_BITMAP *org_bitmap= dbug_tmp_use_all_columns(table, &table->read_set);
   int result= m_write_row(table, buf, fields);
-  dbug_tmp_restore_column_map(table->read_set, org_bitmap);
+  dbug_tmp_restore_column_map(&table->read_set, org_bitmap);
 
   return result;
 }
@@ -432,7 +428,6 @@ int PFS_engine_table::read_row(TABLE *table,
                                unsigned char *buf,
                                Field **fields)
 {
-  my_bitmap_map *org_bitmap;
   Field *f;
   Field **fields_reset;
 
@@ -440,7 +435,7 @@ int PFS_engine_table::read_row(TABLE *table,
   bool read_all= !bitmap_is_clear_all(table->write_set);
 
   /* We internally write to Fields to support the read interface */
-  org_bitmap= dbug_tmp_use_all_columns(table, table->write_set);
+  MY_BITMAP *org_bitmap= dbug_tmp_use_all_columns(table, &table->write_set);
 
   /*
     Some callers of the storage engine interface do not honor the
@@ -452,7 +447,7 @@ int PFS_engine_table::read_row(TABLE *table,
     f->reset();
 
   int result= read_row_values(table, buf, fields, read_all);
-  dbug_tmp_restore_column_map(table->write_set, org_bitmap);
+  dbug_tmp_restore_column_map(&table->write_set, org_bitmap);
 
   return result;
 }
@@ -470,12 +465,10 @@ int PFS_engine_table::update_row(TABLE *table,
                                  const unsigned char *new_buf,
                                  Field **fields)
 {
-  my_bitmap_map *org_bitmap;
-
   /* We internally read from Fields to support the write interface */
-  org_bitmap= dbug_tmp_use_all_columns(table, table->read_set);
+  MY_BITMAP *org_bitmap= dbug_tmp_use_all_columns(table, &table->read_set);
   int result= update_row_values(table, old_buf, new_buf, fields);
-  dbug_tmp_restore_column_map(table->read_set, org_bitmap);
+  dbug_tmp_restore_column_map(&table->read_set, org_bitmap);
 
   return result;
 }
@@ -484,12 +477,10 @@ int PFS_engine_table::delete_row(TABLE *table,
                                  const unsigned char *buf,
                                  Field **fields)
 {
-  my_bitmap_map *org_bitmap;
-
   /* We internally read from Fields to support the delete interface */
-  org_bitmap= dbug_tmp_use_all_columns(table, table->read_set);
+  MY_BITMAP *org_bitmap= dbug_tmp_use_all_columns(table, &table->read_set);
   int result= delete_row_values(table, buf, fields);
-  dbug_tmp_restore_column_map(table->read_set, org_bitmap);
+  dbug_tmp_restore_column_map(&table->read_set, org_bitmap);
 
   return result;
 }
@@ -534,28 +525,28 @@ void PFS_engine_table::get_normalizer(PFS_instr_class *instr_class)
 
 void PFS_engine_table::set_field_long(Field *f, long value)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_LONG);
+  assert(f->real_type() == MYSQL_TYPE_LONG);
   Field_long *f2= (Field_long*) f;
   f2->store(value, false);
 }
 
 void PFS_engine_table::set_field_ulong(Field *f, ulong value)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_LONG);
+  assert(f->real_type() == MYSQL_TYPE_LONG);
   Field_long *f2= (Field_long*) f;
   f2->store(value, true);
 }
 
 void PFS_engine_table::set_field_longlong(Field *f, longlong value)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_LONGLONG);
+  assert(f->real_type() == MYSQL_TYPE_LONGLONG);
   Field_longlong *f2= (Field_longlong*) f;
   f2->store(value, false);
 }
 
 void PFS_engine_table::set_field_ulonglong(Field *f, ulonglong value)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_LONGLONG);
+  assert(f->real_type() == MYSQL_TYPE_LONGLONG);
   Field_longlong *f2= (Field_longlong*) f;
   f2->store(value, true);
 }
@@ -563,7 +554,7 @@ void PFS_engine_table::set_field_ulonglong(Field *f, ulonglong value)
 void PFS_engine_table::set_field_char_utf8(Field *f, const char* str,
                                            uint len)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_STRING);
+  assert(f->real_type() == MYSQL_TYPE_STRING);
   Field_string *f2= (Field_string*) f;
   f2->store(str, len, &my_charset_utf8mb3_bin);
 }
@@ -573,7 +564,7 @@ void PFS_engine_table::set_field_varchar(Field *f,
                                          const char* str,
                                          uint len)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_VARCHAR);
+  assert(f->real_type() == MYSQL_TYPE_VARCHAR);
   Field_varstring *f2= (Field_varstring*) f;
   f2->store(str, len, cs);
 }
@@ -581,7 +572,7 @@ void PFS_engine_table::set_field_varchar(Field *f,
 void PFS_engine_table::set_field_varchar_utf8(Field *f, const char* str,
                                               uint len)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_VARCHAR);
+  assert(f->real_type() == MYSQL_TYPE_VARCHAR);
   Field_varstring *f2= (Field_varstring*) f;
   f2->store(str, len, &my_charset_utf8mb3_bin);
 }
@@ -589,7 +580,7 @@ void PFS_engine_table::set_field_varchar_utf8(Field *f, const char* str,
 void PFS_engine_table::set_field_longtext_utf8(Field *f, const char* str,
                                                uint len)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_BLOB);
+  assert(f->real_type() == MYSQL_TYPE_BLOB);
   Field_blob *f2= (Field_blob*) f;
   f2->store(str, len, &my_charset_utf8mb3_bin);
 }
@@ -597,14 +588,14 @@ void PFS_engine_table::set_field_longtext_utf8(Field *f, const char* str,
 void PFS_engine_table::set_field_blob(Field *f, const char* val,
                                       uint len)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_BLOB);
+  assert(f->real_type() == MYSQL_TYPE_BLOB);
   Field_blob *f2= (Field_blob*) f;
   f2->store(val, len, &my_charset_utf8mb3_bin);
 }
 
 void PFS_engine_table::set_field_enum(Field *f, ulonglong value)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_ENUM);
+  assert(f->real_type() == MYSQL_TYPE_ENUM);
   Field_enum *f2= (Field_enum*) f;
   f2->store_type(value);
 }
@@ -618,14 +609,14 @@ void PFS_engine_table::set_field_timestamp(Field *f, ulonglong value)
 
 void PFS_engine_table::set_field_double(Field *f, double value)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_DOUBLE);
+  assert(f->real_type() == MYSQL_TYPE_DOUBLE);
   Field_double *f2= (Field_double*) f;
   f2->store(value);
 }
 
 ulonglong PFS_engine_table::get_field_enum(Field *f)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_ENUM);
+  assert(f->real_type() == MYSQL_TYPE_ENUM);
   Field_enum *f2= (Field_enum*) f;
   return f2->val_int();
 }
@@ -633,7 +624,7 @@ ulonglong PFS_engine_table::get_field_enum(Field *f)
 String*
 PFS_engine_table::get_field_char_utf8(Field *f, String *val)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_STRING);
+  assert(f->real_type() == MYSQL_TYPE_STRING);
   Field_string *f2= (Field_string*) f;
   val= f2->val_str(NULL, val);
   return val;
@@ -642,7 +633,7 @@ PFS_engine_table::get_field_char_utf8(Field *f, String *val)
 String*
 PFS_engine_table::get_field_varchar_utf8(Field *f, String *val)
 {
-  DBUG_ASSERT(f->real_type() == MYSQL_TYPE_VARCHAR);
+  assert(f->real_type() == MYSQL_TYPE_VARCHAR);
   Field_varstring *f2= (Field_varstring*) f;
   val= f2->val_str(NULL, val);
   return val;
@@ -738,9 +729,10 @@ static bool allow_drop_table_privilege() {
     return false;
   }
 
-  DBUG_ASSERT(thd->lex != NULL);
+  assert(thd->lex != NULL);
   if ((thd->lex->sql_command != SQLCOM_TRUNCATE) &&
-      (thd->lex->sql_command != SQLCOM_GRANT)) {
+      (thd->lex->sql_command != SQLCOM_GRANT) &&
+      (thd->lex->sql_command != SQLCOM_REVOKE)) {
     return false;
   }
 
@@ -1543,11 +1535,11 @@ bool pfs_show_status(handlerton *hton, THD *thd,
       break;
     case 147:
       name= "(filename_hash).count";
-      size= filename_hash.count;
+      size= pfs_filename_hash.count;
       break;
     case 148:
       name= "(filename_hash).size";
-      size= filename_hash.size;
+      size= pfs_filename_hash.size;
       break;
     case 149:
       name= "(host_hash).count";
